"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskController = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const core_2 = require("@sourceloop/core");
const loopback4_authentication_1 = require("loopback4-authentication");
const loopback4_authorization_1 = require("loopback4-authorization");
const models_1 = require("../models");
const repositories_1 = require("../repositories");
const services_1 = require("../services");
const types_1 = require("../types");
const baseUrl = 'tasks';
let TaskController = class TaskController {
    constructor(taskRepo, userTaskRepository, taskWorkflowMapping, camundaService, logger) {
        this.taskRepo = taskRepo;
        this.userTaskRepository = userTaskRepository;
        this.taskWorkflowMapping = taskWorkflowMapping;
        this.camundaService = camundaService;
        this.logger = logger;
    }
    async find(filter) {
        return this.taskRepo.find(filter);
    }
    async findById(id, filter) {
        return this.taskRepo.findById(id, filter);
    }
    async count(where) {
        return this.taskRepo.count(where);
    }
    async mapTaskToWorkflow(taskWorkflowMapping) {
        await this.taskWorkflowMapping.create(taskWorkflowMapping);
    }
    async updateById(id, task) {
        if (task.status) {
            if (task.status === types_1.TaskStatus.Completed) {
                throw rest_1.HttpErrors.BadRequest('Task completion cannot be done through the PATCH API.');
            }
            const updatedUserTask = new models_1.UserTask({ status: task.status });
            await this.userTaskRepository.updateAll(updatedUserTask, {
                taskId: id,
            });
        }
        await this.taskRepo.updateById(id, task);
    }
    async updateAll(tasks, where) {
        if (tasks.status) {
            if (tasks.status === types_1.TaskStatus.Completed) {
                throw rest_1.HttpErrors.BadRequest('Task completion cannot be done through the PATCH API.');
            }
            const existingTasks = await this.taskRepo.find({
                where,
                fields: ['id'],
            });
            if (existingTasks.length) {
                const updatedUserTask = new models_1.UserTask({ status: tasks.status });
                await this.userTaskRepository.updateAll(updatedUserTask, {
                    taskId: {
                        inq: existingTasks.map(task => task.id),
                    },
                });
            }
        }
        return this.taskRepo.updateAll(tasks, where);
    }
    async deleteById(id, cascade = true) {
        if (cascade) {
            const task = await this.taskRepo.findById(id);
            await this.userTaskRepository.deleteAllHard({
                taskId: id,
            });
            this.camundaService
                .deleteProcessInstances([task.externalId])
                .catch(error => {
                this.logger.error(`Failed to delete process instances on Camunda: ${error}`);
            });
        }
        await this.taskRepo.deleteByIdHard(id);
    }
    async delete(where, cascade = true) {
        if (cascade) {
            const tasks = await this.taskRepo.find({
                where,
                fields: ['id', 'deleted', 'externalId'],
            });
            const processInstanceIds = tasks.map(task => task.externalId);
            if (tasks.length) {
                await this.userTaskRepository.deleteAllHard({
                    taskId: {
                        inq: tasks.map(task => task.id),
                    },
                });
                this.camundaService
                    .deleteProcessInstances(processInstanceIds)
                    .catch(error => {
                    this.logger.error(`Failed to delete process instances on Camunda: ${error}`);
                });
            }
        }
        await this.taskRepo.deleteAllHard(where);
    }
};
exports.TaskController = TaskController;
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({
        permissions: ["16003" /* TaskPermssionKey.ViewTask */],
    }),
    (0, rest_1.get)(baseUrl, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [200 /* STATUS_CODE.OK */]: {
                description: 'Array of Task model instances',
                content: {
                    [core_2.CONTENT_TYPE.JSON]: {
                        schema: {
                            type: 'array',
                            items: (0, rest_1.getModelSchemaRef)(models_1.Task, { includeRelations: true }),
                        },
                    },
                },
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.filter(models_1.Task)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "find", null);
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({
        permissions: ["16003" /* TaskPermssionKey.ViewTask */],
    }),
    (0, rest_1.get)(`${baseUrl}/{id}`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [200 /* STATUS_CODE.OK */]: {
                description: 'Task model instance',
                content: {
                    [core_2.CONTENT_TYPE.JSON]: {
                        schema: (0, rest_1.getModelSchemaRef)(models_1.Task, { includeRelations: true }),
                    },
                },
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.path.string('id')),
    tslib_1.__param(1, rest_1.param.filter(models_1.Task, { exclude: 'where' })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "findById", null);
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({
        permissions: ["16003" /* TaskPermssionKey.ViewTask */],
    }),
    (0, rest_1.get)(`${baseUrl}/count`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [200 /* STATUS_CODE.OK */]: {
                description: 'Task model count',
                content: { [core_2.CONTENT_TYPE.JSON]: { schema: repository_1.CountSchema } },
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.where(models_1.Task)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "count", null);
tslib_1.__decorate([
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */),
    (0, loopback4_authorization_1.authorize)({ permissions: ["16006" /* TaskPermssionKey.MapTask */] }),
    (0, rest_1.post)(`${baseUrl}/mapping`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {},
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)({
        content: {
            [core_2.CONTENT_TYPE.JSON]: {
                schema: (0, rest_1.getModelSchemaRef)(models_1.TaskWorkflow, {
                    title: 'TaskWorkFlowMapping',
                }),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "mapTaskToWorkflow", null);
tslib_1.__decorate([
    (0, loopback4_authorization_1.authorize)({
        permissions: ["16011" /* TaskPermssionKey.UpdateTask */],
    }),
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */, {
        passReqToCallback: true,
    }),
    (0, rest_1.patch)(`${baseUrl}/{id}`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [204 /* STATUS_CODE.NO_CONTENT */]: {
                description: 'Task UPDATE by id success',
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.path.string('id')),
    tslib_1.__param(1, (0, rest_1.requestBody)({
        content: {
            'application/json': {
                schema: (0, rest_1.getModelSchemaRef)(models_1.Task, { partial: true }),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, models_1.Task]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "updateById", null);
tslib_1.__decorate([
    (0, loopback4_authorization_1.authorize)({
        permissions: ["16011" /* TaskPermssionKey.UpdateTask */],
    }),
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */, {
        passReqToCallback: true,
    }),
    (0, rest_1.patch)(`${baseUrl}`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [200 /* STATUS_CODE.OK */]: {
                description: 'Tasks UPDATE success',
            },
        },
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)({
        content: {
            'application/json': {
                schema: (0, rest_1.getModelSchemaRef)(models_1.Task, { partial: true }),
            },
        },
    })),
    tslib_1.__param(1, rest_1.param.where(models_1.Task)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [models_1.Task, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "updateAll", null);
tslib_1.__decorate([
    (0, loopback4_authorization_1.authorize)({
        permissions: ["16008" /* TaskPermssionKey.DeleteTask */],
    }),
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */, {
        passReqToCallback: true,
    }),
    (0, rest_1.del)(`${baseUrl}/{id}`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [204 /* STATUS_CODE.NO_CONTENT */]: {
                description: 'Task DELETE by id success',
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.path.string('id')),
    tslib_1.__param(1, rest_1.param.query.boolean('cascade')),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "deleteById", null);
tslib_1.__decorate([
    (0, loopback4_authorization_1.authorize)({
        permissions: ["16008" /* TaskPermssionKey.DeleteTask */],
    }),
    (0, loopback4_authentication_1.authenticate)("bearer" /* STRATEGY.BEARER */, {
        passReqToCallback: true,
    }),
    (0, rest_1.del)(`${baseUrl}`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [204 /* STATUS_CODE.NO_CONTENT */]: {
                description: 'Tasks DELETE success',
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.where(models_1.Task)),
    tslib_1.__param(1, rest_1.param.query.boolean('cascade')),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], TaskController.prototype, "delete", null);
exports.TaskController = TaskController = tslib_1.__decorate([
    tslib_1.__param(0, (0, repository_1.repository)(repositories_1.TaskRepository)),
    tslib_1.__param(1, (0, repository_1.repository)(repositories_1.UserTaskRepository)),
    tslib_1.__param(2, (0, repository_1.repository)(repositories_1.TaskWorkFlowRepository)),
    tslib_1.__param(3, (0, core_1.service)(services_1.CamundaService)),
    tslib_1.__param(4, (0, core_1.inject)(core_2.LOGGER.LOGGER_INJECT)),
    tslib_1.__metadata("design:paramtypes", [repositories_1.TaskRepository,
        repositories_1.UserTaskRepository,
        repositories_1.TaskWorkFlowRepository,
        services_1.CamundaService, Object])
], TaskController);
//# sourceMappingURL=task.controller.js.map